# Duo, A crazy eights game.
# Copyright (C) 2003-2008 The Duo developers.
#
# This file is part of Duo.
#
# Duo is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Automatons set for Duo."""

import logging
import threading

from duo import tools


class AbstractAI(object):
    """Abstract automaton interface."""

    def __init__(self):
        self._thread = None
        self._player = None

    def initialize(self, player):
        """Initialize the automaton."""
        self._player = player

    def start_game(self):
        """Start the game."""
        # TODO: handle the case where game is already running...
        #       ... or the previous thread didn't finished.
        self._thread = threading.Thread(name=self._player.name, target=self.run)
        self._thread.start()

    def stop_game(self):
        """Interrupt the game."""
        raise NotImplementedError, \
              '%s shall define stop_game' % self.__class__.__name__

    def get_player(self):
        """Retrieve the protocol player this automaton is attached to."""
        return self._player

    def run(self):
        raise NotImplementedError, \
              '%s shall define run' % self.__class__.__name__

    def on_auth(self, payload, emitter, args):
        """Hook called when a new player authenticates himself."""
        raise NotImplementedError, \
              '%s shall define onAuth' % self.__class__.__name__

    def on_hand(self, payload, emitter, args):
        """Hook called when new cards are distributed."""
        raise NotImplementedError, \
              '%s shall define onHand' % self.__class__.__name__

    def on_turn(self, payload, emitter, args):
        """Hook called when a new player should play."""
        raise NotImplementedError, \
              '%s shall define onTurn' % self.__class__.__name__

    def on_heap(self, payload, emitter, args):
        """Hook called when a new card is set on top of the heap."""
        raise NotImplementedError, \
              '%s shall define onHeap' % self.__class__.__name__

    def on_deal(self, payload, emitter, args):
        """Hook called when a player receives one or more cards."""
        raise NotImplementedError, \
              '%s shall define onDeal' % self.__class__.__name__

    def on_error(self, payload, emitter, args):
        """Hook called on any protocol or game error."""
        raise NotImplementedError, \
              '%s shall define onError' % self.__class__.__name__

    def on_winner(self, payload, emitter, args):
        """Hook called on game end."""
        raise NotImplementedError, \
              '%s shall define onError' % self.__class__.__name__


def load(name, alt_name):
    """Load an automaton.

    An automaton is defined by its name. An alternate name can be used
    to avoid having multiple players with the same name.
    """
    try:
        # TODO: manage name collisions on transport layer.
        module = tools.import_recursively('duo.ai.' + name)
        return module.AI()
    except Exception, all_e:
        logging.getLogger('duo.ai').error('load %s: %s' % (name, all_e))
